# -*- ruby -*-
# $Id: canvas.rb,v 1.2 2002/07/17 18:47:29 ttate Exp $

require 'fltk'

module Fltk
  class Canvas < Fltk::Widget
    include Fltk::Drawable

    attr_accessor :scroll, :grid

    def initialize(*args)
      super(*args)
      @items = []
      @focused_item = nil
      @selected_item = nil
      @scroll = nil
      @grid = 10
    end

    def add_item(item)
      @items.push(item)
    end

    def delete_item(item)
      @items.delete(item)
    end

    def draw
      clear
      @items.each{|item|
	if( item == @focused_item )
	  item.swap_color{
	    item.draw
	  }
	else
	  item.draw
	end
      }
    end

    def handle(event)
      eb = Fltk.event_button
      ek = Fltk.event_key
      ex = Fltk.event_x
      ey = Fltk.event_y
      if( @scroll )
	sx = @scroll.xposition
	sy = @scroll.yposition
      else
	sx = 0
	sy = 0
      end
      
      case event
      when Fltk::PUSH
	cur_time = Time.now
	if( @pre_event_time && (cur_time - @pre_event_time < 0.5) )
	  @pre_event_time = nil
	  return @selected_item.callback(ex,ey,eb)
	else
	  @pre_event_time = cur_time
	end

	@focused_item = nil
	@items.each{|item|
	  if( item.under_mouse?(ex + sx, ey + sy) )
	    @focused_item = item
	    break
	  end
	}
	if( @focused_item )
	  # move the focused item to the end of the item list.
	  @items.delete(@focused_item)
	  @items.push(@focused_item)

	  if( eb == 1 )
	    @selected_item = @focused_item
	    @selected_rx   = ex
	    @selected_ry   = ey
	    redraw
	    return true
	  else
	    r = @focused_item.callback(ex,ey,eb)
	    redraw
	    return r
	  end
	end
	@pre_event_time = Time.now
	return false
      when Fltk::DRAG
	if( @selected_item )
	  @selected_item.move(@selected_rx,
			      @selected_ry,
			      ex,
			      ey)
	  @selected_rx = ex
	  @selected_ry = ey
	end
	redraw
	return true
      else
	return false
      end
    end
  end
end

module Fltk
  class Canvas
    class CanvasItem
      def initialize(canvas, lstyle = nil, lwidth = nil,
		     color1 = nil, color2 = nil, fill = nil)
	@line_style = lstyle || Fltk::SOLID
	@line_width = lwidth || 1
	@color1 = color1 || Fltk::BLACK
	@color2 = color2 || Fltk::RED
	@fill  = fill  || false
	@range  = 5
	@canvas = canvas
	@canvas.add_item(self)
      end

      def grid(*vals)
	return vals.collect{|v| v.divmod(@canvas.grid)[0] * @canvas.grid}
      end

      def swap_color
	@color1,@color2 = [@color2,@color1]
	yield
	@color1,@color2 = [@color2,@color1]
      end

      def set_style
	Fltk.line_style(@line_style, @line_width)
	Fltk.color(@color1)
	if( block_given? )
	  yield
	  Fltk.line_style(0)
	  Fltk.color(0)
	end
      end

      def draw
      end

      def callback(x, y, button)
	return false
      end
      
      def move(x1,y1,x2,y2)
      end

      def under_mouse?(x, y)
	return false
      end
    end
  end
end

require 'fltk/canvas/line'
require 'fltk/canvas/circle'
require 'fltk/canvas/rect'
require 'fltk/canvas/polygon'
require 'fltk/canvas/label'
require 'fltk/canvas/ellipse'
