# -*- ruby -*-
# $Id: line.rb,v 1.1 2001/11/17 12:36:53 ttate Exp $

module Fltk
class Canvas
  class Line < CanvasItem
    def initialize(canvas, vs, lstyle = nil, lwidth = nil,
		   color1 = nil, color2 = nil, fill = nil)
      super(canvas, lstyle, lwidth, color1, color2, fill)
      @vertexes = vs
    end

    def move(x1, y1, x2, y2)
      dx = x2 - x1
      dy = y2 - y1
      @vertexes.each{|v|
	vx,vy = v
	r = (x1 - vx) ** 2 + (y1 - vy) ** 2
	if( r <= @range ** 2 )
	  v[0] += dx
	  v[1] += dy
	  return
	end
      }
      @vertexes.each{|v|
	v[0] += dx
	v[1] += dy
      }
    end

    def under_mouse?(x, y)
      for i in 0..(@vertexes.size - 2)
	v1,v2 = @vertexes[i,2]
	x1,y1 = v1
	x2,y2 = v2

	# if the pointer is on outside of the line,...
	dy = y2 - y1
	dx = x2 - x1
	if( dx == 0 )
	  if( y1 < y2 )
	    next if( !(y1 <= y && y <= y2) )
	  else
	    next if( !(y2 <= y && y <= y1) )
	  end
	  dd = (x1 - x) ** 2
	elsif( dy == 0 )
	  if( x1 < x2 )
	    next if( !(x1 <= x && x <= x2) )
	  else
	    next if( !(x2 <= x && x <= x1) )
	  end
	  dd = (y1 - y) ** 2
	else
	  if( x1 < x2 )
	    next if( !(x1 <= x && x <= x2) )
	  else
	    next if( !(x2 <= x && x <= x1) )
	  end
	  if( y1 < y2 )
	    next if( !(y1 <= y && y <= y2) )
	  else
	    next if( !(y2 <= y && y <= y1) )
	  end
	  # make the form 'y = a x + b'.
	  # a and b must be a floating value since the gradient
	  # may be lower than 1.
	  a  = dy.to_f / dx.to_f
	  b  = y1.to_f - a * x1.to_f
	  # convert to the normal form 'xx x + yy y + k = 0'.
	  xx = -a
	  yy = 1.0
	  kk = -b
	  # dd = distance^2
	  dd = ((xx * x + yy * y + kk) ** 2) / ((xx ** 2) + (yy ** 2))
	end
	if( dd.to_i <= @range ** 2 )
	  return true
	end
      end
      return false
    end

    def draw
      for i in 0..(@vertexes.size - 2)
	v1,v2 = @vertexes[i,2]
	set_style{
	  @canvas.line(*grid(v1[0], v1[1], v2[0], v2[1]))
	}
      end
    end
  end
end
end
