require 'rubyfltk.so'

module Fltk
  module Drawable
    def color(c)
      Fltk::color(c)
    end

    def clear
      color(Fltk::GRAY)
      rectf(0, 0, self.w, self.h)
    end

    def fill(c)
      color(c)
      rectf(0, 0, self.w, self.h)
    end

    def push_clip(x, y, w, h)
      Fltk::push_clip(realx(x), realy(y), w, h)
    end

    def push_no_clip
      Fltk::push_no_clip
    end

    def pop_clip
      Fltk::pop_clip
    end

    def not_clipped(x, y, w, h)
      Fltk::not_clipped(realx(x), realy(y), w, h)
    end
    alias not_clipped? not_clipped

    def clip_box(x, y, w, h)
      Fltk::clip_box(realx(x), realy(y), w, h)
    end

    def line_style(style, width = 0, dashes = nil)
      Fltk::line_style(style, width, dashes)
    end

    def point(x,y)
      Fltk::point(realx(x), realy(y))
    end
    
    def line(x1,y1,x2,y2)
      Fltk::line(realx(x1), realy(y1), realx(x2), realy(y2))
    end
    
    def rect(x,y,w,h)
      Fltk::rect(realx(x), realy(y), w, h)
    end
	
    def rectf(x,y,w,h)
      Fltk::rectf(realx(x), realy(y), w, h)
    end
	
    def pie(x,y,w,h,a1,a2)
      Fltk::pie(realx(x), realy(y), w, h, a1, a2)
    end

    def arc(x,y,w,h,a1,a2)
      Fltk::arc(realx(x), realy(y), w, h, a1, a2)
    end

    def circle(x,y,r)
      Fltk::circle(x,y,r)
    end

    def begin_line()
      Fltk::begin_line()
      if( block_given? )
	yield
	Fltk::end_line()
      end
    end

    def end_line()
      Fltk::end_line()
    end

    def begin_loop()
      Fltk::begin_loop()
      if( block_given? )
	yield
	Fltk::end_loop()
      end
    end

    def end_loop()
      Fltk::end_loop()
    end

    def begin_polygon()
      Fltk::begin_polygon()
      if( block_given? )
	yield
	Fltk::end_polygon
      end
    end

    def end_polygon()
      Fltk::end_polygon()
    end

    def begin_complex_polygon()
      Fltk::begin_complex_polygon()
      if( block_given? )
	yield
	Fltk::end_complex_polygon()
      end
    end

    def end_complex_polygon()
      Fltk::end_complex_polygon()
    end

    def vertex(x, y)
      Fltk::vertex(realx(x), realy(y))
    end

    def draw_text(text,x,y,w=nil,h=nil,align=nil)
      if( w && h && align )
	Fltk::draw(text,realx(x),realy(y),w,h,align)
      elsif( w && h && !align )
	Fltk::draw(text,realx(x),realy(y),w,h)
      elsif( !w && !h && !align )
	Fltk::draw(text,realx(x),realy(y))
      else
	raise(ArgumentError, "wrong argument")
      end
    end
    alias text_draw draw_text

    def text_width(text, n = nil)
      if( n )
	Fltk.width(text, n)
      else
	Fltk.width(text)
      end
    end

    def draw_pixmap(xpm, x, y, color = nil)
      if( color )
	Fltk.draw_pixmap(xpm, realx(x), realy(y), color)
      else
	Fltk.draw_pixmap(xpm, realx(x), realy(y))
      end
    end

    def realx(x)
      return self.x + x
    end
    
    def realy(y)
      return self.y + y
    end
  end
end
